#import "common.typ"
#set page(
  paper: "a4",
  margin: 2cm,
  flipped: true,
)
#set par(justify: true)
#set text(
  font: "Linux Libertine",
  size: 11pt,
)

#show heading.where(level: 2): it => text.with(size: 16pt, weight: "bold")([#v(6pt) #it #v(4pt)])

#let design(it) = { 
  par[#it]
}

#let redflag(name, desc) = {
  par[#text(weight: "bold")[#name:] #desc]
}

#common.titlebar(
  title: "A Philosophy of Software Design",
  author: "John Ousterhout"
)
#columns( 3, gutter: 1cm, [

  == Design Principles
  Here are the most important software design principles discussed in this book:

  #enum(
    design[Complexity is incremental: you have to sweat the small stuff.],
    design[Working code isn’t enough.],
    design[Make continual small investments to improve system design.],
    design[Modules should be deep.],
    design[Interfaces should be designed to make the most common usage as simple as possible.],
    design[It’s more important for a module to have a simple interface than a simple implementation.],
    design[General-purpose modules are deeper.],
    design[Separate general-purpose and special-purpose code.],
    design[Different layers should have different abstractions.],
    design[Pull complexity downward.],
    design[Define errors (and special cases) out of existence.],
    design[Design it twice.],
    design[Comments should describe things that are not obvious from the code.],
    design[Software should be designed for ease of reading, not ease of writing.],
    design[The increments of software development should be abstractions, not features.],
  )

  == Red Flags
  Here are a few of of the most important red flags discussed in this book.
  The presence of any of these symptoms in a system suggests that there is a problem with the system’s design:

  #redflag("Shallow Module", "The interface for a class or method isn’t much simpler than its implementation.")
  #redflag("Information Leakage", "A design decision is reflected in multiple modules.")
  #redflag("Temporal Decomposition", "The code structure is based on the order in which operations are executed, not on information hiding.")
  #redflag("Overexposure", "An API forces callers to be aware of rarely used features in order to use commonly used features.")
  #redflag("Pass-Through Method", "A method does almost nothing except pass its arguments to another method with a similar signature.")
  #redflag("Repetition", "A nontrivial piece of code is repeated over and over.")
  #redflag("Special-General Mixture", "Special-purpose code is not cleanly separated from general purpose code.")
  #redflag("Conjoined Methods", "Two methods have so many dependencies that its hard to understand the implementation of one without understanding the implementation of the other.")
  #redflag("Comment Repeats Code", "All of the information in a comment is immediately obvious from the code next to the comment.")
  #redflag("Implementation Documentation Contaminates Interface", "An interface comment describes implementation details not needed by users of the thing being documented.")
  #redflag("Vague Name", "The name of a variable or method is so imprecise that it doesn’t convey much useful information.")
  #redflag("Hard to Pick Name", "It is difficult to come up with a precise and intuitive name for an entity.")
  #redflag("Hard to Describe", "In order to be complete, the documentation for a variable or method must be long.")
  #redflag("Nonobvious Code", "The behavior or meaning of a piece of code cannot be understood easily.")
])

